#!/bin/bash

# Path to ESBMC binary
ESBMC_PATH=/home/jmorse/phd/esbmc/esbmc/esbmc
# A temporary file to pipe output into for later examination
TMPFILE=`mktemp`

if test $# -ne 2; then
        echo "usage: esbmcwrapper.sh dirname filename" >&2
        exit 1
fi

# Two arguments: the directory name and the path to the file to check.
dirname=$1
filename=$2

# Options we might run with.
NORMALOPTS=' --unwind 6 --no-unwinding-assertions --partial-loops --64 --no-assertions -D_Bool=int --no-bounds-check --no-pointer-check --no-div-by-zero-check --error-label ERROR'
MEMSAFETYOPTS=' --unwind 6 --no-unwinding-assertions --partial-loops --64 --no-assertions -D_Bool=int --memory-leak-check'

# There are two modes we might run in: Normal, and memsafety. The first is just
# a general set of options that work well for all testcases, the second is
# options specific to checking the memory safety category. The drivers64
# catagory receives no special options.

# Decide what options to run with.
if test "$dirname" = "memsafety"; then
	THEOPTS=$MEMSAFETYOPTS
else
	THEOPTS=$NORMALOPTS
fi

run_bmc()
{
	$ESBMC_PATH $THEOPTS $filename 2>&1 | tee $TMPFILE
}

# Run ESBMC in k-induction mode for half the time
$ESBMC_PATH $THEOPTS $filename --timeout 450s --k-induction --k-step 100 --memlimit 15g 2>&1 | tee $TMPFILE

# Did we produce a concrete answer?
grep "VERIFICATION SUCCESSFUL" $TMPFILE > /dev/null
UNREACH=$?
grep "VERIFICATION FAILED" $TMPFILE > /dev/null
REACH=$?
grep "VERIFICATION UNKNOWN" $TMPFILE > /dev/null
UNKNOWN=$?

# If we produced a concrete answer, use that. Otherwise, run in BMC mode for the
# rest of the allocated time. All pthread tests will be rejected by
# ESBMC in k-induction mode.
if test $UNKNOWN -eq 0; then
	# No answer could be produced; run in BMC mode.
	run_bmc
elif test $UNREACH -eq 0; then
	# We've decided that the property is unreachable. Report and exit.
	echo "true"
	exit 0
elif test $REACH -eq 0; then
	# We've decided that the property is reachable. Report and exit.
	echo "false"
	exit 0
else
	# Something bad happened; default to running BMC mode.
	run_bmc
fi

# Examine the output from BMC mode. Note that order is important; multithreaded
# output will print SUCCESS repeatedly, and will possibly print FAILED if a
# counterexample interleaving is found. So check for presence of FAILED before
# SUCCESS.

grep "VERIFICATION SUCCESSFUL" $TMPFILE > /dev/null
UNREACH=$?
grep "VERIFICATION FAILED" $TMPFILE > /dev/null
REACH=$?

if test $REACH -eq 0; then
	# We've decided that the property is reachable
	echo "false"
	exit 0
elif test $UNREACH -eq 0; then
	# We've decided that the property is unreachable
	echo "true"
	exit 0
else
	echo "error"
	exit 0
fi
